from django.utils.functional import curry, SimpleLazyObject

from django_oauth_twitter import ACCESS_KEY, REQUEST_KEY, USERINFO_KEY
from django_oauth_twitter.utils import fail_whale, get_user_info

from oauth.oauth import OAuthToken
import simplejson
import twitter


class SessionMiddleware(object):
    def process_request(self, request):
        """Import the Twitter token from the Session into the Request."""
        request.twitter_access_token = None
        request.twitter_request_token = None
        request.twitter_userinfo = None
        if ACCESS_KEY in request.session:
            token_str = request.session[ACCESS_KEY]
            token = OAuthToken.from_string(token_str)
            request.twitter_access_token = token
            userinfo = SimpleLazyObject(curry(cached_user_info,
                                              request, token))
            request.twitter_userinfo = userinfo
        if REQUEST_KEY in request.session:
            token_str = request.session[REQUEST_KEY]
            request.twitter_request_token = OAuthToken.from_string(token_str)


def cached_user_info(request, token):
    userinfo = None
    try:
        userinfo = fail_whale(get_user_info)(token)
    except:
        pass
    if userinfo is None:
        # Look for a cached copy in the session
        if USERINFO_KEY in request.session:
            userinfo_dict = simplejson.loads(request.session[USERINFO_KEY])
            userinfo = twitter.User.NewFromJsonDict(userinfo_dict)
    else:
        request.session[USERINFO_KEY] = userinfo.AsJsonString()
    return userinfo

def remove_tokens(request):
    if ACCESS_KEY in request.session:
        del request.session[ACCESS_KEY]
        request.twitter_access_token = None
    if REQUEST_KEY in request.session:
        del request.session[REQUEST_KEY]
        request.twitter_request_token = None
    if USERINFO_KEY in request.session:
        del request.session[USERINFO_KEY]
        request.twitter_userinfo = None

def set_access_token(request, access_token):
    request.session[ACCESS_KEY] = access_token.to_string()
    request.twitter_access_token = access_token

def set_request_token(request, request_token):
    request.session[REQUEST_KEY] = request_token.to_string()
    request.twitter_request_token = request_token



from urllib2 import HTTPError, URLError

from django.conf import settings
from django.utils.functional import update_wrapper


try:
    from oauthtwitter import OAuthApi
except AttributeError:
    # oauthtwitter has a nasty bug, where it imports oauth incorrectly.
    import oauth
    import oauth.oauth
    oauth.__dict__.update((k, v) for k, v in oauth.oauth.__dict__.iteritems()
                          if not (k.startswith('__') and k.endswith('__')))
    from oauthtwitter import OAuthApi


def TwitterApi(token=None):
    """
    Returns an OAuthApi object, given an optional `token`.
    """
    # Use the default consumer key and secret from settings.
    return OAuthApi(consumer_key=settings.TWITTER_CONSUMER_KEY,
                    consumer_secret=settings.TWITTER_CONSUMER_SECRET,
                    access_token=token)


def get_user_info(access_token):
    return TwitterApi(access_token).GetUserInfo()

def fail_whale(f):
    MAX_TRIES = 3
    def wrapper(*args, **kwargs):
        for tries in range(MAX_TRIES - 1):
            try:
                return f(*args, **kwargs)
            except HTTPError, e:
                if e.code != 503:
                    # Retry when Service Temporarily Unavailable
                    raise
            except URLError, e:
                errno = getattr(e.reason, 'args', [None])[0]
                if errno != 8:
                    # Retry when EOF occurred in violation of protocol
                    raise
        return f(*args, **kwargs)
    return update_wrapper(wrapper, f)



from cgi import parse_qs
from urllib import urlencode
from urllib2 import HTTPError
from urlparse import urlsplit, urlunsplit

from django.conf import settings
from django.contrib.auth import login
from django.core.urlresolvers import get_callable, NoReverseMatch, reverse
from django.conf.urls.defaults import patterns, url
from django.db import IntegrityError
from django.http import HttpResponseRedirect, HttpResponse
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.utils.translation import ugettext_lazy as _

from django_oauth_twitter.forms import RegistrationForm
from django_oauth_twitter.middleware import (cached_user_info,
                                             remove_tokens,
                                             set_access_token,
                                             set_request_token)
from django_oauth_twitter.models import TwitterUser
from django_oauth_twitter.utils import fail_whale, TwitterApi

from oauth.oauth import OAuthToken


class OAuthTwitter(object):
    RegistrationForm=RegistrationForm

    def __init__(self,
                 auto_create_user=False,
                 association_url=None,
                 no_thanks_url='django.contrib.auth.views.logout',
                 registration_url=None,
                 signed_in_url='/'):
        self.auto_create_user = auto_create_user
        if association_url is None:
            self.association_url = LazyReverse(self.associate)
        else:
            self.association_url = LazyReverse(association_url)
        self.no_thanks_url = LazyReverse(no_thanks_url)
        if registration_url is None:
            self.registration_url = LazyReverse(self.register)
        else:
            self.registration_url = LazyReverse(registration_url)
        self.signed_in_url = LazyReverse(signed_in_url)

    def get_urls(self):
        urlpatterns = patterns('',
            url(r'^callback/', self.callback, name='twitter_callback'),
            url(r'^signin/', self.signin, name='twitter_signin'),
        )
        if self.association_url.location == self.associate:
            # Add ^associate/ location if we're meant to handle association.
            urlpatterns += patterns('',
                url(r'^associate/', self.association_url.location,
                    name='twitter_associate')
            )
        if self.registration_url.location == self.register:
            # Add ^register/ location if we're meant to handle registration.
            urlpatterns += patterns('',
                url(r'^register/', self.registration_url.location,
                    name='twitter_register')
            )
        return urlpatterns
    urls = property(get_urls)

    def associate(self, request,
                  template="django_oauth_twitter/associate.html",
                  post_associate=None,
                  post_remove=None):
        user = request.user
        try:
            if not user.is_authenticated():
                return self._redirect_to_login(request)
            if request.method == 'POST':
                if request.POST.get('action') == 'remove':
                    try:
                        twitter_id = int(request.POST.get('twitter_id'))
                    except ValueError:
                        twitter_id = None
                    try:
                        twitter_user = user.twitter
                        if twitter_user.twitter_id == twitter_id:
                            twitter_user.delete()
                        del user._twitter_cache
                        remove_tokens(request)
                        if post_remove is not None:
                            post_remove()
                    except TwitterUser.DoesNotExist:
                        pass
            else:
                try:
                    twitter_user = request.user.twitter
                except TwitterUser.DoesNotExist:
                    twitter_user = None
                if request.GET.get('associated') and twitter_user is not None:
                    if post_associate is not None:
                        post_associate()
                    return HttpResponseRedirect(request.path)
            return render_to_response(template,
                                      context_instance=RequestContext(request))
        except Exception, e:
            import traceback
            traceback.print_exc()
            raise


    def callback(self, request):
        """
        Gets the oauth_token from Twitter and signs in a User.

        Note: You must set this callback URL in Twitter's Application
        Details page.  Twitter ignores OAuth's oauth_callback option.
        """
        def access_token(request_token):
            try:
                return TwitterApi(request_token).getAccessToken()
            except HTTPError, e:
                if e.code == 401:
                    return HttpResponseRedirect(reverse(self.signin))
                raise
        # Ensure that the user came in through signin().
        request_token = request.twitter_request_token
        if request_token is None:
            return self._redirect_to_login(request=request)
        # Ensure that the session's token matches Twitter's token.
        if request_token.key != request.GET.get('oauth_token'):
            remove_tokens(request)
            return self._redirect_to_login(request=request)
        # Save the access token in the session.
        access_token = fail_whale(access_token)(request_token)
        set_access_token(request, access_token)
        userinfo = cached_user_info(request, access_token)
        user = request.user
        if user.is_anonymous():
            # Find the User by the access token.
            user = self._authenticate(userinfo=userinfo)
            if user is None:
                # New user
                return self._on_new_user(request=request)
            return self._login_and_redirect(request=request, user=user)
        else:
            try:
                TwitterUser.objects.create_twitter_user(
                    user=user,
                    access_token=access_token,
                    userinfo=userinfo
                )
            except IntegrityError:
                # This User already has a linked Twitter account.
                return HttpResponseRedirect(self.association_url)
            # Add ?associated=1 to self.association_url
            s, n, p, query, f = urlsplit(str(self.association_url))
            query_dict = parse_qs(query)
            query_dict['associated'] = '1'
            association_url = urlunsplit((s, n, p, urlencode(query_dict), f))
            return HttpResponseRedirect(association_url)

    def signin(self, request):
        """Redirects a user to the Twitter authorization page."""
        def get_signin_url(request):
            twitter = TwitterApi()
            request_token = twitter.getRequestToken()
            set_request_token(request, request_token)
            return twitter.getSigninURL(request_token)
        return HttpResponseRedirect(fail_whale(get_signin_url)(request))

    def register(self, request,
                 template='django_oauth_twitter/register.html'):
        # Ensure that the user has an access token
        access_token = request.twitter_access_token
        if access_token is None:
            return self._redirect_to_login(request=request)
        if request.user.is_anonymous():
            userinfo = request.twitter_userinfo
            screen_name = userinfo.screen_name
            if request.method == "POST":
                # Register the user
                form = self.RegistrationForm(request.POST,
                                             access_token=access_token,
                                             userinfo=userinfo)
                if form.is_valid():
                    user = form.save()
                    return self._login_and_redirect(request=request,
                                                    user=user)
            else:
                # Pre-fill with the user's Twitter screen name
                form = self.RegistrationForm(initial={'username': screen_name})
            return render_to_response(template,
                                      {'form': form,
                                       'no_thanks': self.no_thanks_url,
                                       'screen_name': screen_name},
                                      RequestContext(request))
        return HttpResponseRedirect(self.signed_in_url)

    def _authenticate(self, userinfo):
        """
        Returns user if `userinfo` is associated with a known user.
        Otherwise, returns None.
        """
        if userinfo is None:
            return None
        try:
            user = TwitterUser.objects.get(twitter_id=userinfo.id).user
        except TwitterUser.DoesNotExist:
            return None
        return user

    def _create_user(self, request):
        """
        Creates a User from their Twitter account and redirects to `url`.

        Afterwards, `login_and_redirect` to `url`.
        """
        user = None
        if request.user.is_anonymous():
            # Create the User
            access_token = request.twitter_access_token
            userinfo = request.twitter_userinfo
            form = self.RegistrationForm({'username': userinfo.screen_name},
                                         access_token=access_token,
                                         userinfo=userinfo)
            if form.is_valid():
                user = form.save()
            else:
                return HttpResponseRedirect(self.registration_url)
        # Login and redirect
        return self._login_and_redirect(request=request, user=user)

    def _login(self, request, user):
        # Nasty but necessary - annotate user and pretend it was the regular
        # auth backend. This is needed so django.contrib.auth.get_user works:
        user.backend = 'django.contrib.auth.backends.ModelBackend'
        login(request, user)

    def _login_and_redirect(self, request, user):
        """
        Login `user` with `access_token` and redirects to `signed_in_url`.

        If the `request.user` is already logged in, don't clobber the
        login with `user`.  The `request.user` has to log out first.
        """
        if user is not None and request.user.is_anonymous():
            # Login
            self._login(request, user)
            # Update the access_token
            twitter = user.twitter
            save = False
            if twitter.update_access_token(request.twitter_access_token):
                save = True
            if twitter.update_userinfo(request.twitter_userinfo):
                save = True
            if save:
                twitter.save()
        # Redirect
        return HttpResponseRedirect(self.signed_in_url)

    def _on_new_user(self, request):
        """
        Handles an AnonymousUser who has just authenticated with Twitter.

        If `self.auto_create_user` is True, creates a new User with a
        username matching their Twitter screen name.  Otherwise,
        redirects the user into the registration process.
        """
        if self.auto_create_user:
            return self._create_user(request)
        return HttpResponseRedirect(self.registration_url)

    def _redirect_to_login(self, request):
        """Redirect to settings.LOGIN_URL."""
        return HttpResponseRedirect(settings.LOGIN_URL)


class LazyReverse(object):
    def __init__(self, location):
        self.location = location
        self.url = None

    def __str__(self):
        if callable(self.location) or '/' not in self.location:
            self.url = reverse(self.location)
        else:
            self.url = self.location
        return self.url



from django.contrib.auth.models import User
from django.db import models
from django.utils.translation import ugettext_lazy as _

from oauth.oauth import OAuthToken
import simplejson
import twitter

from django_oauth_twitter.utils import get_user_info


class TwitterUserManager(models.Manager):
    @staticmethod
    def _access_token(access_token, userinfo=None):
        if userinfo is None:
            userinfo = get_user_info(access_token)
        attributes = {'access_token_str': access_token,
                      'twitter_id': userinfo.id,
                      'userinfo_json': userinfo.AsJsonString()}
        return (attributes, userinfo)

    def create_twitter_user(self, user, access_token, userinfo=None):
        """
        Returns a new TwitterUser from `user` and `access_token`.

        If `userinfo` is provided, it will use that instead of
        fetching recent Twitter user info using `access_token`.
        """
        attributes, userinfo = self._access_token(access_token, userinfo)
        return self.create(user=user, **attributes)

    def update_or_create(self, user, access_token, userinfo=None):
        """
        Returns a (TwitterUser, created) tuple from `user` and `access_token`.

        `created` is True when the TwitterUser had to be created.

        If `userinfo` is provided, it will use that instead of
        fetching recent Twitter user info using `access_token`.
        """
        attributes, userinfo = self._access_token(access_token, userinfo)
        obj, created = self.get_or_create(user=user,
                                          defaults=attributes)
        if not created:
            save = False
            if obj.update_access_token(access_token):
                save = True
            if obj.update_userinfo(userinfo):
                save = True
            if save:
                obj.save()
        user._twitter_cache = obj
        return obj, created


class TwitterUser(models.Model):
    user = models.OneToOneField(User, unique=True, verbose_name=_('user'),
                                related_name='twitter')
    twitter_id = models.IntegerField(unique=True)
    access_token_str = models.TextField()
    userinfo_json = models.TextField(blank=True)
    objects = TwitterUserManager()

    def __unicode__(self):
        return self.userinfo().screen_name

    def get_access_token(self):
        if self.access_token_str:
            return OAuthToken.from_string(self.access_token_str)
        return None

    def set_access_token(self, value):
        if hasattr(value, 'to_string'):
            self.access_token_str = value.to_string()
        else:
            self.access_token_str = value

    access_token = property(get_access_token, set_access_token)

    def update_access_token(self, access_token):
        if self.access_token != access_token:
            self.access_token = access_token
            return True

    def userinfo(self):
        if self.userinfo_json:
            userinfo_dict = simplejson.loads(self.userinfo_json)
            userinfo = twitter.User.NewFromJsonDict(userinfo_dict)
        else:
            userinfo = self.update_userinfo()
        return userinfo

    def update_userinfo(self, userinfo=None):
        if userinfo is None:
            userinfo = get_user_info(self.access_token)
        userinfo_json = userinfo.AsJsonString()
        if self.userinfo_json != userinfo_json:
            self.userinfo_json = userinfo_json
            return userinfo

    def get_screen_name(self):
        return self.userinfo().screen_name
    
    screen_name = property(get_screen_name)



from django.utils.functional import SimpleLazyObject

from twitter import User

def twitter_userinfo(request):
    twitter_userinfo = request.twitter_userinfo
    if not twitter_userinfo:
        return {}
    return {'twitter_userinfo': twitter_userinfo}



from django.contrib.auth.models import User
from django.forms import ModelForm

from django_oauth_twitter.models import TwitterUser
from django_oauth_twitter.utils import get_user_info

class RegistrationForm(ModelForm):
    class Meta:
        model = User
        fields = ('username',)

    def __init__(self, *args, **kwargs):
        self.access_token = kwargs.pop('access_token', None)
        self.userinfo = kwargs.pop('userinfo', None)
        initial = kwargs.get('initial', None)
        if initial is not None and 'username' in initial:
            if User.objects.filter(username=initial['username']):
                # User already exists in the system, so don't prefill the
                # username.
                del initial['username']
        return super(RegistrationForm, self).__init__(*args, **kwargs)

    def save(self):
        user = super(RegistrationForm, self).save(commit=False)
        user.set_unusable_password()
        user.save()
        # Associate it with a Twitter account.
        TwitterUser.objects.update_or_create(user=user,
                                             access_token=self.access_token,
                                             userinfo=self.userinfo)
        return user



from __future__ import with_statement

import os

from django.conf import settings
from django.contrib.auth.models import AnonymousUser, User
from django.test import TestCase
from django.core.urlresolvers import NoReverseMatch, reverse

from mocker import Mocker, ANY, MATCH

from oauth.oauth import OAuthToken

from django_oauth_twitter import ACCESS_KEY, REQUEST_KEY, USERINFO_KEY
from django_oauth_twitter.middleware import (cached_user_info,
                                             set_access_token,
                                             set_request_token)
from django_oauth_twitter.models import TwitterUser
from django_oauth_twitter.test_urls import oauthtwitter
from django_oauth_twitter.views import LazyReverse, OAuthTwitter
from django_oauth_twitter.utils import TwitterApi, get_user_info

TOKEN = OAuthToken.from_string('oauth_token=a&oauth_token_secret=b')


class MiddlewareTest(TestCase):
    def setUp(self):
        self.mocker = Mocker()

    def test_set_access_token(self):
        request = self.mocker.mock()
        request.session[ACCESS_KEY] = TOKEN.to_string()
        request.twitter_access_token = TOKEN
        with self.mocker:
            set_access_token(request, TOKEN)

    def test_set_request_token(self):
        request = self.mocker.mock()
        request.session[REQUEST_KEY] = TOKEN.to_string()
        request.twitter_request_token = TOKEN
        with self.mocker:
            set_request_token(request, TOKEN)


class UtilsTest(TestCase):
    def setUp(self):
        settings.TWITTER_CONSUMER_KEY = 'KEY'
        settings.TWITTER_CONSUMER_SECRET = 'SECRET'
        self.mocker = Mocker()

    def test_twitter_api(self):
        from oauthtwitter import OAuthApi
        self.assertTrue(isinstance(TwitterApi(), OAuthApi))
        self.assertTrue(isinstance(TwitterApi(token=''), OAuthApi))

    def test_get_user_info(self):
        Api = self.mocker.replace(TwitterApi)
        Api(TOKEN).GetUserInfo()
        self.mocker.result('GetUserInfo')
        with self.mocker:
            self.assertEqual(get_user_info(TOKEN), 'GetUserInfo')


class LazyReverseTest(TestCase):
    urls = 'django_oauth_twitter.test_urls'

    def test_str(self):
        self.assertEqual(str(LazyReverse('http://google.com/')),
                         'http://google.com/')
        self.assertEqual(str(LazyReverse('/home/')), '/home/')
        self.assertEqual(str(LazyReverse('home')), '/home/')
        self.assertRaises(NoReverseMatch, str, LazyReverse('unknown'))
        self.assertEqual(str(LazyReverse(oauthtwitter.register)),
                         '/twitter/register/')


class OAuthTwitterTest(TestCase):
    urls = 'django_oauth_twitter.test_urls'

    def setUp(self):
        self.mocker = Mocker()
        self.password = User.objects.create_user('password', '', 'password')
        self.twitter = User.objects.create_user('twitter', '', 'twitter')
        json = '{"screen_name": "twitter"}'
        self.twitter_user = TwitterUser.objects.create(user=self.twitter,
                                                       twitter_id=1,
                                                       userinfo_json=json)
        settings.TEMPLATE_DIRS = (os.path.join(__file__, 'templates'),)

    def test_association_anonymous(self):
        # Ensure we need a login.
        response = self.client.get(reverse(oauthtwitter.associate))
        self.assertEqual(response['Location'],
                         'http://testserver%s' % settings.LOGIN_URL)

    def test_association_user_add(self):
        # User adds association.
        self.assertRaises(TwitterUser.DoesNotExist,
                          TwitterUser.objects.get, user=self.password)
        self.assertTrue(self.client.login(username=self.password.username,
                                          password=self.password.username))
        Api = self.mocker.replace(TwitterApi)
        Api(ANY).getAccessToken()
        self.mocker.result(TOKEN)
        userinfo = self.mocker.mock()
        userinfo.id
        self.mocker.result('2')
        userinfo.AsJsonString()
        self.mocker.result('{}')
        self.mocker.count(1, 2)
        Api(ANY).GetUserInfo()
        self.mocker.result(userinfo)
        with self.mocker:
            # User goes off to Twitter and gets redirected back to callback
            session = self.client.session
            session[REQUEST_KEY] = TOKEN.to_string()
            session.save()
            response = self.client.get(reverse(oauthtwitter.callback),
                                       {'oauth_token': TOKEN.key},
                                       follow=True)
            self.assertEqual(response.redirect_chain,
                [('http://testserver/twitter/associate/?associated=1', 302),
                 ('http://testserver/twitter/associate/', 302)]
            )
            self.assertEqual(response.status_code, 200)

    def test_association_user_remove(self):
        # User removes association.
        self.assertRaises(TwitterUser.DoesNotExist,
                          TwitterUser.objects.get, user=self.password)
        self.assertTrue(self.client.login(username=self.password.username,
                                          password=self.password.username))
        response = self.client.post(reverse(oauthtwitter.associate),
                                    {'action': 'remove',
                                     'twitter_id': '1'})
        self.assertEqual(response.status_code, 200)
        self.assertRaises(TwitterUser.DoesNotExist,
                          TwitterUser.objects.get, user=self.password)
        response = self.client.post(reverse(oauthtwitter.associate))
        self.assertEqual(response.status_code, 200)
        self.assertRaises(TwitterUser.DoesNotExist,
                          TwitterUser.objects.get, user=self.password)

    def test_association_associated_add(self):
        # Associated adds association.
        self.assertTrue(self.twitter.twitter)
        self.assertTrue(self.client.login(username=self.twitter.username,
                                          password=self.twitter.username))
        Api = self.mocker.replace(TwitterApi)
        Api(ANY).getAccessToken()
        self.mocker.result(TOKEN)
        userinfo = self.mocker.mock()
        userinfo.id
        self.mocker.result('2')
        userinfo.AsJsonString()
        self.mocker.result('{}')
        self.mocker.count(1, 2)
        Api(ANY).GetUserInfo()
        self.mocker.result(userinfo)
        with self.mocker:
            # User goes off to Twitter and gets redirected back to callback
            session = self.client.session
            session[REQUEST_KEY] = TOKEN.to_string()
            session.save()
            response = self.client.get(reverse(oauthtwitter.callback),
                                       {'oauth_token': TOKEN.key})
            self.assertEqual(response['Location'],
                             'http://testserver/twitter/associate/')

    def test_association_associated_remove(self):
        # Associated removes association.
        self.assertTrue(self.twitter.twitter)
        self.assertTrue(self.client.login(username=self.twitter.username,
                                          password=self.twitter.username))
        response = self.client.post(reverse(oauthtwitter.associate))
        self.assertEqual(response.status_code, 200)
        self.assertTrue(TwitterUser.objects.get(user=self.twitter))
        response = self.client.post(reverse(oauthtwitter.associate),
                                    {'action': 'remove',
                                     'twitter_id': '1'})
        self.assertEqual(response.status_code, 200)
        self.assertRaises(TwitterUser.DoesNotExist,
                          TwitterUser.objects.get, user=self.twitter)


    def test_callback(self):
        # User goes directly to the callback page.
        response = self.client.get(reverse(oauthtwitter.callback))
        self.assertEqual(response['Location'],
                         'http://testserver%s' % settings.LOGIN_URL)
        # User goes directly to the callback page. but tampers with oauth_token
        response = self.client.get(reverse(oauthtwitter.callback),
                                   {'oauth_token': 'bad'})
        self.assertEqual(response['Location'],
                         'http://testserver%s' % settings.LOGIN_URL)
        # User comes in through Twitter OAuth, but tampers with oauth_token
        o = self.mocker.patch(oauthtwitter)
        request = self.mocker.mock()
        request.GET
        self.mocker.result({'oauth_token': 'bad'})
        request.session
        self.mocker.result({})
        self.mocker.count(1, None)
        request.twitter_request_token
        self.mocker.result(TOKEN)
        # New user comes in through Twitter OAuth
        request.GET
        self.mocker.result({'oauth_token': 'a'})
        request.twitter_request_token
        self.mocker.result(TOKEN)
        Api = self.mocker.replace(TwitterApi)
        Api(TOKEN).getAccessToken()
        self.mocker.result(TOKEN)
        request.twitter_access_token = ANY
        request.user
        self.mocker.result(AnonymousUser())
        userinfo = self.mocker.mock()
        cui = self.mocker.replace(cached_user_info)
        cui(request, TOKEN)
        self.mocker.result(userinfo)
        o._authenticate(userinfo=userinfo)
        self.mocker.result(None)
        # Existing user comes in through Twitter OAuth
        request.GET
        self.mocker.result({'oauth_token': 'a'})
        request.twitter_request_token
        self.mocker.result(TOKEN)
        Api(TOKEN).getAccessToken()
        self.mocker.result(TOKEN)
        request.twitter_access_token = ANY
        request.user
        self.mocker.result(User.objects.get(username='password'))
        cui(request, TOKEN)
        self.mocker.result(userinfo)
        userinfo.id
        self.mocker.result(3)
        userinfo.AsJsonString()
        self.mocker.result('{}')
        with self.mocker:
            response = o.callback(request)
            self.assertEqual(response['Location'], settings.LOGIN_URL)
            response = o.callback(request)
            self.assertEqual(response['Location'],
                             reverse(oauthtwitter.register))
            response = o.callback(request)
            self.assertEqual(response['Location'],
                             '/twitter/associate/?associated=1')

    def test_signin(self):
        response = self.client.get(reverse(oauthtwitter.signin))
        prefix = 'http://twitter.com/oauth/authenticate?'
        self.assertTrue(response['Location'].startswith(prefix),
                        response['Location'])

    def test_register(self):
        # User goes directly to the register page.
        response = self.client.get(reverse(oauthtwitter.register))
        self.assertTrue(response['Location'], settings.LOGIN_URL)
        # User is logged in
        self.assertTrue(self.client.login(username='password',
                                          password='password'))
        response = self.client.get(reverse(oauthtwitter.register))
        self.assertTrue(response['Location'], '/home/')
        # New user comes in through Twitter
        request = self.mocker.mock()
        request.user
        self.mocker.result(AnonymousUser())
        self.mocker.count(1, None)
        RC = self.mocker.replace('django.template.RequestContext')
        RC(ANY)
        self.mocker.result({})
        request.twitter_access_token
        self.mocker.result(TOKEN)
        userinfo = self.mocker.mock()
        userinfo.screen_name
        self.mocker.result('unknown')
        request.twitter_userinfo
        self.mocker.result(userinfo)
        request.method
        self.mocker.result('GET')
        # New user posts form through Twitter
        request.twitter_access_token
        self.mocker.result(TOKEN)
        userinfo.id
        self.mocker.result(2)
        userinfo.screen_name
        self.mocker.result('unknown')
        userinfo.AsJsonString()
        self.mocker.result('{}')
        request.twitter_userinfo
        self.mocker.result(userinfo)
        request.method
        self.mocker.result('POST')
        request.POST
        self.mocker.result({'username': 'unknown'})
        o = self.mocker.patch(oauthtwitter)
        o._login_and_redirect(request=request, user=ANY)
        self.mocker.result('New user')
        with self.mocker:
            response = oauthtwitter.register(request)
            self.assertEqual(response.status_code, 200)
            response = o.register(request)
            self.assertEqual(response, 'New user')
            self.assertTrue(User.objects.get(username='unknown'))

    def test_authenticate(self):
        # Associated Twitter user
        twitter = self.mocker.mock()
        twitter.id
        self.mocker.result(1)
        # Unknown Twitter uesr
        unknown = self.mocker.mock()
        unknown.id
        self.mocker.result(2)
        with self.mocker:
            self.assertEqual(oauthtwitter._authenticate(twitter),
                             User.objects.get(username="twitter"))
            self.assertEqual(oauthtwitter._authenticate(unknown), None)

    def test_create_user(self):
        o = self.mocker.patch(oauthtwitter)
        request = self.mocker.mock()
        # User already logged in
        request.user.is_anonymous()
        self.mocker.result(False)
        o._login_and_redirect(request=request, user=None)
        self.mocker.result('Logged in')
        # Anonymous user with new User
        request.user.is_anonymous()
        self.mocker.result(True)
        request.twitter_access_token
        self.mocker.result(TOKEN)
        unknown = self.mocker.mock()
        unknown.id
        self.mocker.result(2)
        unknown.screen_name
        self.mocker.result("unknown")
        unknown.AsJsonString()
        self.mocker.result('{}')
        request.twitter_userinfo
        self.mocker.result(unknown)
        o._login_and_redirect(request=request,
                              user=MATCH(lambda x: isinstance(x, User)))
        self.mocker.result('New user')
        # Anonymous user with existing User
        request.user.is_anonymous()
        self.mocker.result(True)
        request.twitter_access_token
        self.mocker.result(TOKEN)
        twitter = self.mocker.mock()
        twitter.screen_name
        self.mocker.result("twitter")
        request.twitter_userinfo
        self.mocker.result(twitter)
        with self.mocker:
            response = o._create_user(request)
            self.assertEqual(response, 'Logged in')
            response = o._create_user(request)
            self.assertEqual(response, 'New user')
            response = o._create_user(request)
            self.assertEqual(response['Location'], '/twitter/register/')

    def test_login_and_redirect(self):
        o = self.mocker.patch(oauthtwitter)
        request = self.mocker.mock()
        # User already logged in
        twitter = User.objects.get(username='twitter')
        request.user
        self.mocker.result(twitter)
        # No User to log in
        True
        # Login a Twitter user
        o._login(request, twitter)
        request.user
        self.mocker.result(AnonymousUser())
        request.twitter_access_token
        self.mocker.result(TOKEN)
        userinfo = self.mocker.mock()
        userinfo.AsJsonString()
        self.mocker.result('{}')
        request.twitter_userinfo
        self.mocker.result(userinfo)
        with self.mocker:
            response = o._login_and_redirect(request, True)
            self.assertEqual(response['Location'], '/home/')
            response = o._login_and_redirect(request, None)
            self.assertEqual(response['Location'], '/home/')
            response = o._login_and_redirect(request, twitter)
            self.assertEqual(response['Location'], '/home/')
            self.assertEqual(str(twitter.twitter.access_token), str(TOKEN))
            self.assertEqual(twitter.twitter.userinfo_json, '{}')

    def test_on_new_user(self):
        # With auto_create_user
        request = self.mocker.mock()
        a = self.mocker.patch(OAuthTwitter(auto_create_user=True))
        a._create_user(request)
        self.mocker.result('auto_create')
        # Without
        b = self.mocker.patch(oauthtwitter)
        with self.mocker:
            response = a._on_new_user(request)
            self.assertEqual(response, 'auto_create')
            response = b._on_new_user(request)
            self.assertEqual(response['Location'], '/twitter/register/')

    def test_redirect_to_login(self):
        response = oauthtwitter._redirect_to_login(None)
        self.assertEqual(response['Location'], settings.LOGIN_URL)



from django.contrib import admin

from django_oauth_twitter.models import TwitterUser

class TwitterUserAdmin(admin.ModelAdmin):
    list_display = ('user', 'twitter_id', 'screen_name')
    search_fields = ('user', 'twitter_id')
    ordering = ('user',)

admin.site.register(TwitterUser, TwitterUserAdmin)



#!/usr/bin/env python
from setuptools import setup

setup(
    name='django-oauth-twitter',
    version='1.0',
    description=('A Django application that lets you associate Twitter '
                 'accounts with your User accounts'),
    long_description=(
"""
Features include:

* Uses django.contrib.auth.User
* Lets Twitter users sign in before registering.
* Optional auto-creation of Users from Twitter screen-names
* Lets Users link and unlink Twitter accounts.
"""
    ),
    author='Akoha Inc.',
    author_email='adminmail@akoha.com',
    url='http://bitbucket.org/akoha/django-oauth-twitter/',
    packages=['django_oauth_twitter'],
    package_dir={'django_oauth_twitter': 'django_oauth_twitter'},
    package_data={
        'django_oauth_twitter': ['templates/django_oauth_twitter/*.html'],
    },
    requires=['oauth', 'oauth_python_twitter'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
    ],
    zip_safe=True,
)



from django.conf.urls.defaults import *

from django_oauth_twitter.views import OAuthTwitter

oauthtwitter = OAuthTwitter(signed_in_url='/home/')

def home(request):
    pass

urlpatterns = patterns('',
    url(r'^twitter/', include(oauthtwitter.urls)),
    url(r'^home/', home, name='home'),
    url(r'^logout/', 'django.contrib.auth.views.logout'),
)

